# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Sync Server
#
# The Initial Developer of the Original Code is the Mozilla Foundation.
# Portions created by the Initial Developer are Copyright (C) 2010
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Tarek Ziade (tarek@mozilla.com)
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****
""" SQL Mappers
"""
from services.auth.sqlmappers import users

from sqlalchemy.ext.declarative import declarative_base, Column
from sqlalchemy import Integer, String, Text, BigInteger


_Base = declarative_base()
tables = []
MAX_TTL = 2100000000

# mapper defined in services.auth
tables.append(users)


class Collections(_Base):
    __tablename__ = 'collections'
    # XXX add indexes
    userid = Column(Integer, primary_key=True, nullable=False)
    collectionid = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(32), nullable=False)


collections = Collections.__table__
tables.append(collections)


class WBO(_Base):
    __tablename__ = 'wbo'
    __table_args__ = {'mysql_engine': 'InnoDB',
                      'mysql_charset': 'latin1'}

    id = Column(String(64), primary_key=True, autoincrement=False)
    username = Column(Integer, primary_key=True, nullable=False)
    collection = Column(Integer, primary_key=True, nullable=False,
                        default=0)
    parentid = Column(String(64))
    predecessorid = Column(String(64))
    sortindex = Column(Integer)
    modified = Column(BigInteger)
    payload = Column(Text)
    payload_size = Column(Integer, nullable=False, default=0)
    ttl = Column(Integer, default=MAX_TTL)


wbo = WBO.__table__


# preparing the sharded tables
class _WBOBase(object):
    id = Column(String(64), primary_key=True, autoincrement=False)
    username = Column(Integer, primary_key=True, nullable=False)
    collection = Column(Integer, primary_key=True, nullable=False,
                        default=0)
    parentid = Column(String(64))
    predecessorid = Column(String(64))
    sortindex = Column(Integer)
    modified = Column(BigInteger)
    payload = Column(Text)
    payload_size = Column(Integer, nullable=False, default=0)
    ttl = Column(Integer, default=MAX_TTL)


_SHARDS = {}


def get_wbo_table_byindex(index):
    if index not in _SHARDS:
        args = {'__tablename__': 'wbo%d' % index,
                '__table_args__':
                     {'mysql_engine': 'InnoDB',
                      'mysql_charset': 'latin1'}}
        klass = type('WBO%d' % index, (_WBOBase, _Base), args)
        _SHARDS[index] = klass.__table__
    return _SHARDS[index]


def get_wbo_table(user_id, shardsize=100):
    return get_wbo_table_byindex(int(user_id) % shardsize)


def get_wbo_table_name(user_id, shardsize=100):
    return 'wbo%d' % (int(user_id) % shardsize)
